/*
 * Fichier : Affepaper.c
 * Auteur : Pascal Chour - www.pascalchour.fr
 * Date : 15 septembre 2019
 * (c) Pascal Chour
 *
 * Principe d'affichage
 * MOY = Moyenne courante sur 24h (si temps coul > 24h)
 * MIN = Moyenne du minimum courant sur 24h
 * MAX = moyenne du maximum courant sur 24h OU si alarme en cours, valeur max enregistre (affiche en rouge). On sort du mode alarme en pressant sur le bouton
 * Si Alarme en cours, on affiche ALARME (blanc sur rouge). Sinon, on affiche NORMAL (blanc sur noir)
 * Batterie : indicateur mesure sur le processeur et indicateur mesure sur la batterie
 */

#include "Affepaper.h"
#include "utilitaires.h"
#include "PC_Epd_154_low_memory.h"
#include "stdbool.h"
#include "stdint.h"
#include "string.h"
#include "common.h"



// Variables globales internes au module
uint16_t AffJr_Courant  = 0; // Jour affich
uint32_t AffMoy_Courant = 0; // Moyenne affiche
uint16_t AffMax_Courant = 0; // Max affich
uint16_t Vbat = 0;           // Tension batterie
bool AffAlarme_Courant = false;
const char Tsep[] =   "\x21\x0";
const char JR[]  =    "\x23\x28\x20\x20\x21\x20\x0";
const char MAX[] =    "\x25\x22\x29\x20\x21\x20\x0";
const char MOY[] =    "\x25\x27\x2A\x20\x21\x20\x0";
const char NORMAL[] = "\x26\x27\x28\x25\x22\x24\x0";
const char ALARM[] =  "\x22\x24\x22\x28\x25\x0";


/*
 * Donne la tension de la batterie en centivolts. La conversion en bargraph se fait ailleurs.
 * Ex: 367 = 3,67V
 * Vproc = tension au niveau du processeur, VBat = tension au niveau de la batterie
 * N'est normalement appele qu'une fois toutes les 24h
 */
void TensionBatterie(uint16_t * Vproc, uint16_t * Vbat)
{
   uint32_t adcResult;                      // Pour lecture ADC
// Mesure de la tension processeur
    ADCCTL0 &= ~(ADCON + ADCENC + ADCSC);   // Dsactive le convertisseur
    ADCIE &= 0;                             // Invalide toutes les interruptions
    ADCIFG &= 0;                            // plus d'interruptions en cours
    ADCCTL0 = ADCSHT_2 | ADCON;             // 16 ADCCLK cycles pour la conversion (ADCSHT_2) + mise en route du convertisseur (ADCON).
    ADCCTL1 = ADCSHP;                       // signal is sourced from the sampling timer (ADCSHP), ADC clock source select = MODOSC, The sample input signal is not inverted, ADC clock divide = 1, Single-channel single-conversion
    ADCCTL2 = ADCRES_1;                     // ADC predivider = 1, ADC resolution = 10-bit (ADCRES_1), ADC data read-back format = unsigned, ADC sampling rate =200ksps
    ADCMCTL0 = ADCINCH_13 | ADCSREF_0;      // 000b = {V R+ = AVCC and V R = AVSS }, Select reference = 13 = 1,5V reference voltage
    PMMCTL2 |= INTREFEN;                    // ACtive la rfrence interne (note : PMM dj dverrouill lors de l'initialisation gnrale)
    while(!(PMMCTL2 & REFGENRDY));          // Boucle jusqu' ce que les rfrences internes soit stabilises
    ADCCTL0 |= ADCENC | ADCSC;              // Sampling and conversion start
    while (!(ADCIFG & ADC_COMPLETED_INTERRUPT_FLAG)){SleepMS(2);}
    ADCIFG &= ~ADC_COMPLETED_INTERRUPT_FLAG;
    adcResult = ADCMEM0;

    // Tension = (1023 * 150) / ADCMEM0
    // 150 pour 150cV. 1023 pour 10bits de rsolution
    if (adcResult > 0) {adcResult = ((unsigned long)1023 * (unsigned long)150) / adcResult;} else {adcResult = 0;}
    * Vproc = adcResult;


    // Mesure de la tension aux bornes de la batterie
    GPIO_setOutputHighOnPin(PORT_BAT_EN, PIN_BAT_EN); // active la mesure de la batterie
    SleepMS(10);
    ADCCTL0 &= ~(ADCON + ADCENC + ADCSC);    // Dsactive le convertisseur
    ADCIE &= 0;                              // Invalide toutes les interruptions
    ADCIFG &= 0;                             // plus d'interruptions en cours
    ADCCTL1 = ADC_SAMPLEHOLDSOURCE_SC+(ADC_CLOCKDIVIDER_1 & ADCDIV_7)+ADC_CLOCKSOURCE_ADCOSC+ADCSHP + ADC_SINGLECHANNEL; //Setup the Sample-and-Hold Source
    ADCCTL2 = (ADC_CLOCKDIVIDER_1 & ADCPDIV_3) + ADCRES;     //Set Clock Pre-Divider and Default resolution to 10-bits
    ADCCTL0 |= ADCON + ADC_CYCLEHOLD_16_CYCLES;              //Reset the ADCON bit to enable the ADC Module
    ADCCTL0 &= ~(ADCSHT_15 + ADCMSC);        //Reset and Set CB Control 0 Bits
    ADCMCTL0 = ADC_INPUT_A3+ADC_VREFPOS_INT+ADC_VREFNEG_AVSS;
    ADCCTL0 |= ADCENC | ADCSC;
    while (!(ADCIFG & ADC_COMPLETED_INTERRUPT_FLAG)){SleepMS(2);}
    ADCIFG &= ~ADC_COMPLETED_INTERRUPT_FLAG;
    adcResult = ADCMEM0;
    if (adcResult > 0) {adcResult = (adcResult * (unsigned long)150 * (unsigned long)FACTOR_BATTERIE) / (unsigned long)102300;} else {adcResult = 0;}
    *Vbat = adcResult;
    GPIO_setOutputLowOnPin(PORT_BAT_EN, PIN_BAT_EN); // desactive la mesure de la batterie
    ADCCTL0 &= ~ADCON;
    ADCIFG = 0;
    PMMCTL2 &= ~INTREFEN;                   // Dsactive la rfrence interne
}

/*
 * Formatte un uint16 en caractre pour le epaper cadr  droite sur 4 caractres termin par un null. La valeur est
 * concatne  T qui doit tre suffisamment grand et initialis !
 * La valeur max est 9999
 * En entre :  T, chaine de caractre en prfixe de la valeur
 *              Valeur : valeur  convertir
 * En sortie : T = T + Valeur en caractre epaper. T = T+"----" si dpassement
 */

void Uint16epaper(uint16_t Valeur, char T[]) {
    int16_t i;
    uint16_t j;
    j = strlen(T);
    T[j+4]='\0';
    if (Valeur > 9999) {
        strcat(T,"\x29\x29\x29\x29"); // "XXXX"
    } else {
        if (Valeur == 0) {
            strcat(T,"\x20\x20\x20\x2C"); // "   0"
        } else {
            i = 3;
            j = strlen(T);
            while (Valeur > 0) {
                T[j+i] = Valeur - ((Valeur/10)*10) + 0x2C;
                Valeur = Valeur/10;
                i = i-1;
            }
            while (i >= 0) {
                T[j+i] = '\x20';
                i = i-1;
            }
        }
    }
}

/* Cette routine fournit le texte d'affichage de la batterie.
 * En entre : tension processeur, tension batterie, True = banc noir, false = banc rouge, offset (voir commentaire), T = tableau de symboles  afficher
 *
 */
void AffBatterie(uint16_t Vproc, uint16_t Vbat, bool BancNoir) {
uint16_t offset1;
char T1[10];
char T2[11];
    //          0   1   2   3   4   5   6   7   8
    strcpy(T1,"\x21\x22\x23\x20\x20\x21\x22\x23\x00");
    T2[0] = '\x0';
    Uint16epaper(Vproc, T2);
    strncat(T2,"\x20\x0",1);
    Uint16epaper(Vbat, T2);
    if (Vproc > V_PROC_SEG_1) {T1[2] = '\x26';}
    if (Vproc > V_PROC_SEG_2) {T1[1] = '\x25';}
    if (Vproc > V_PROC_SEG_3) {T1[0] = '\x24';}
    if (Vbat > V_BAT_SEG_1) {T1[7] = '\x26';}
    if (Vbat > V_BAT_SEG_2) {T1[6] = '\x25';}
    if (Vbat > V_BAT_SEG_3) {T1[5] = '\x24';}
    offset1 = 0;
    if (BancNoir) {
        if ((Vbat <= V_BAT_SEG_1) && (Vproc <= V_PROC_SEG_1)) {T1[0] = '\x0';}
        else if ((Vbat <= V_BAT_SEG_1) && (Vproc > V_PROC_SEG_1)) {T1[3] = '\x0'; T2[5] = '\x0';}
        else if ((Vbat > V_BAT_SEG_1) && (Vproc <= V_PROC_SEG_1)) {offset1 = 5;}
    } else { // banc rouge
        if ((Vbat > V_BAT_SEG_1) && (Vproc > V_PROC_SEG_1)) {T1[0] = '\x0'; T2[0] = '\x0';}
        else if ((Vbat > V_BAT_SEG_1) && (Vproc <= V_PROC_SEG_1)) {T1[3] = '\x0'; T2[5] = '\x0';}
        else if ((Vproc > V_PROC_SEG_1) && (Vbat <= V_BAT_SEG_1)) {offset1 = 5;}
    }
    EpdSessionWriteString(1+offset1,&T2[offset1], 15,NOTINVERTED, NOTINVERTED);
    EpdSessionGoToLineRel(2,UNCOLORED); // goto from current position to current position +2 lines
    EpdSessionWriteString(2+offset1,&T1[offset1], 20,NOTINVERTED, NOTINVERTED);
}


/*
 * Cette routine permet d'afficher les informations (efface alarme, moyenne, min, max, affiche batterie) sur l'afficheur
 * e-paper.
 * En entre :  ValideAlarme true => on efface l'alarme, ensuite, on laisse le programme dcider. faux = on laisse le programme dcider
 *              AffForce = true => affiche toutes les valeurs et met  jour les xxx_Courant. Faux => ne met  jour que les valeurs qui ont changes ou qui sont imposes
 * Comme il n'est pas plus coteux d'afficher que de se dplacer, on raffraichit systmatiquement les donnes jusqu' la dernire  afficher
 * Traitement particulier pour l'alarme : si une alarme est en cours, la valeur max est affiche en rouge et elle n'est change que si la nouvelle valeur max
 * est suprieure  celle mmorise.
 */
void AffDonnees(bool ValideAlarme, bool AffForce) {
uint16_t Niveau = 0;    // lignes qu'il faut crire.
uint16_t Niveau_Courant;
uint32_t AffMoy = 0;    // Moyenne courante
uint16_t i;
uint16_t Vproc;         // Tension processeur
char T[20];
    for (i = 0; i < 25;i++) {AffMoy = countMoyenneHeure[i]+AffMoy;}
    if (NbMinutes24h > 0) {AffMoy = AffMoy/NbMinutes24h;} else {AffMoy = 0;}
    if (CountMax < AffMoy) {CountMax = AffMoy;} // Ca peut arriver lorsqu'on rinitialise l'alarme. C'est mieux que le Max soit >=  la moyenne !

    // Traitement des alarmes (valeurs du Max et AffAlarme). Un peu sordide...
    if (ValideAlarme) { // Si on valide l'alarme
        if (AffAlarme_Courant) { // et qu'une alarme tait en cours, alors
            if (CountMax < SeuilAlarme) { // Si la valeur du max courant est infrieure au seuil d'alarme
                Niveau = 2; // Alors, il n'y a plus d'alarme et il faut afficher la 1re ligne (panneau Rouge -> panneau Noir) ainsi que le max courant
                AffMax_Courant = CountMax; // on prend le max en entre
                AffAlarme_Courant = false; // Il n'y a plus d'alarme
            } // Si la valeur passe en paramtre est suprieure ou gale au max courant, alors, il y a toujours une alarme aprs la validation. On laisse le panneau en rouge
        } // S'il n'y avait pas d'alarme en cours, la validation est inutile. On ne fait rien  ce stade
    } else { // Pas de validation de l'alarme. On vrifie l'tat courant
        if (AffAlarme_Courant) { // Si une alarme est en cours
            if (CountMax > AffMax_Courant) { // Si le max est > au max courant,
                AffMax_Courant = CountMax; // on prend le max en entre
                Niveau = 2; // et on indique qu'il faut rafficher le max
            } // Sinon, on laisse dans l'tat
        } else { // Pas d'alarme en cours
            if (CountMax >= SeuilAlarme) { // Si le Max en entre est suprieur au seuil d'alarme
                AffMax_Courant = CountMax; // Alors, on met  jour le max courant
                Niveau = 2; // Il faut rafficher le max
                AffAlarme_Courant = true; // et on indique qu'il y a une alarme
            } else { // Le seuil d'alarme n'est pas atteint
                if (CountMax != AffMax_Courant) { // Si la valeur a chang depuis le dernier affichage
                    Niveau = 2;
                    AffMax_Courant = CountMax;
                }
            }
        }
    }
    if (AffForce) {
        Niveau = 5;
    } else {
        if (AffMoy_Courant != AffMoy) {Niveau = max16(3,Niveau);}
        if (AffJr_Courant != NbJours) {Niveau = max16(4,Niveau);}
    }
    AffJr_Courant = NbJours;
    AffMoy_Courant = AffMoy;
    if (AffForce) {TensionBatterie(&Vproc, &Vbat);}

    Niveau_Courant = Niveau;
    if (Niveau_Courant > 0) { // Il y a quelque chose  afficher
        EpdInit(); // on rveille l'afficheur
        if (Niveau_Courant >= 5) {EpdSessionClearDisplay();}
        EpdSessionStart(BLACK);      // Selection BLACK bank, current position = 0
        if (!AffAlarme_Courant) {
            // Alarme
            EpdSessionGoToLineRel(13,COLORED); // goto from current position to current position +12 lines and fill the background in black color
            EpdSessionWriteString(3, NORMAL, 15, INVERTED, INVERTED); // write a white color string on a black background color. 3 ' ' before the text
            EpdSessionGoToLineRel(13,COLORED); // goto from current position to current position +12 lines and fill the background in black color
        } else {
            EpdSessionGoToLineRel(26+EpdSessionGetFontHeight(15),UNCOLORED); // Goto line 24+Height font 40 + 2*Height font 4 => Erase the top of the screen
        }
        Niveau_Courant = Niveau_Courant-1;
        if (Niveau_Courant > 0) {
            // Maximum
            EpdSessionGoToLineRel(2,COLORED);  // draw two black lines
            EpdSessionWriteString(5,Tsep, 4, NOTINVERTED, NOTINVERTED);  // write Tsep with font 4
            EpdSessionWriteString(5,Tsep, 4, NOTINVERTED, NOTINVERTED);  // write Tsep with font 4
            if (!AffAlarme_Courant) {
                strcpy(T,MAX);
                if (CountMax == 0) {Uint16epaper(0xFFFF,T);} else {Uint16epaper(CountMax,T);}
                EpdSessionWriteString(1,T, 15, NOTINVERTED, NOTINVERTED);
            } else {
                EpdSessionWriteString(5,"\x21\0", 15, NOTINVERTED, NOTINVERTED);
            }
            Niveau_Courant = Niveau_Courant-1;

            if (Niveau_Courant > 0) {
                // Moyenne
                EpdSessionWriteString(5,Tsep, 4, NOTINVERTED, NOTINVERTED);
                EpdSessionGoToLineRel(1,COLORED);
                EpdSessionWriteString(5,Tsep, 4, NOTINVERTED, NOTINVERTED);
                EpdSessionWriteString(5,Tsep, 4, NOTINVERTED, NOTINVERTED);
                strcpy(T,MOY);
                Uint16epaper(AffMoy,T);
                EpdSessionWriteString(1,T, 15, NOTINVERTED, NOTINVERTED); // write T with font 15
                Niveau_Courant = Niveau_Courant-1;
                if (Niveau_Courant > 0) {
                    // NbJours
                    EpdSessionWriteString(5,Tsep, 4, NOTINVERTED, NOTINVERTED);
                    EpdSessionGoToLineRel(1,COLORED);
                    EpdSessionWriteString(5,Tsep, 4, NOTINVERTED, NOTINVERTED);
                    EpdSessionWriteString(5,Tsep, 4, NOTINVERTED, NOTINVERTED);
                    strcpy(T,JR);
                    Uint16epaper(NbJours,T);
                    EpdSessionWriteString(1,T, 15, NOTINVERTED, NOTINVERTED);
                    Niveau_Courant = Niveau_Courant-1;
                    if (Niveau_Courant > 0) {
                        // Batterie
                        EpdSessionWriteString(5,Tsep, 4, NOTINVERTED, NOTINVERTED);
                        EpdSessionGoToLineRel(1,COLORED);
                        EpdSessionGoToLineAbs(150,UNCOLORED);  // goto line (absolute)
                        AffBatterie(Vproc, Vbat, true);
                        Niveau_Courant = Niveau_Courant-1;
                    }
                }
            }
        }
    }
    if (Niveau > 0) { // Il y a quelque chose  afficher
        EpdSessionStart(RED);  // Select the Red banc
        if (AffAlarme_Courant) {
            // Alarme
            EpdSessionGoToLineRel(13,COLORED); // goto from current position to current position +12 lines and fill the background in red color
            EpdSessionWriteString(3,ALARM, 15, INVERTED, INVERTED); // write a white color string on a black background color
            EpdSessionGoToLineRel(13,COLORED); // goto from current position to current position +12 lines and fill the background in red color
            if (Niveau >= 2) {
                EpdSessionGoToLineRel(2+2*EpdSessionGetFontHeight(4),UNCOLORED);
                strcpy(T,MAX);
                Uint16epaper(CountMax,T);
                EpdSessionWriteString(1,T, 15, NOTINVERTED, NOTINVERTED);
            }
        }
        EpdSessionGoToLineAbs(150,UNCOLORED);
        if (Niveau >= 5) {
            AffBatterie(Vproc, Vbat, false);
        }

        EpdSessionClose();
        EpdSleep();
    }

}

